"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Validator = void 0;
const api_1 = require("./api");
class Validator {
    constructor(rules, allowNull, allowUndefined) {
        this.rules = rules;
        this.allowNull = allowNull;
        this.allowUndefined = allowUndefined;
    }
    withRule(rule) {
        return new this.constructor(this.rules.concat(rule), this.allowNull, this.allowUndefined);
    }
    optional() {
        return new this.constructor(this.rules, true, true);
    }
    orNull() {
        return new this.constructor(this.rules, true, this.allowUndefined);
    }
    orUndefined() {
        return new this.constructor(this.rules, this.allowNull, true);
    }
    describe() {
        if (this.rules.length > 0) {
            let message = this.rules.map(rule => rule.describe()).join(' and ');
            if (this.allowNull) {
                message += ' or null';
            }
            if (this.allowUndefined) {
                message += ' or undefined';
            }
            return message;
        }
        return 'any';
    }
    test(value) {
        return this.getFailingRules(value).next().done === true;
    }
    check(value, name) {
        const first = this.getFailingRules(value).next();
        if (!first.done) {
            throw this.getRuleError(first.value, name);
        }
    }
    testAll(value, name) {
        return Array.from(this.getFailingRules(value), rule => this.getRuleError(rule, name));
    }
    *getFailingRules(value) {
        if (!(this.allowNull && value === null) && !(this.allowUndefined && value === undefined)) {
            for (const rule of this.rules) {
                const result = rule.test(value);
                if (!result) {
                    yield rule;
                }
            }
        }
    }
    getRuleError(rule, name) {
        let message = `Expected ${rule.describe()}`;
        if (name) {
            message += ` for "${name}"`;
        }
        return new api_1.ValidationError(message, rule, name);
    }
}
exports.Validator = Validator;
//# sourceMappingURL=Validator.js.map